/**
 * This file contains the listener functions which can be assigned to the
 * coprus_analysis client. So that the incoming data/status informations will
 * be handled.
 */

/**
 * Recieves a corpus analysis connected signal via socket.io.
 */
function recieveConnected(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('Connected!')
      console.info('corpus_analysis_init: Client recieving connected codes',
                   'codes via socket.on');
      console.info(`corpus_analysis_init: ${response.code} - ${response.msg}`);
      console.info('corpus_analysis_init: Initialization succeeded');
      console.info(response);
      client.notifyView('connected');
      console.groupEnd();
      // get meta data immediately
      client.getMetaData();
    } else {
      let errorText = `Error ${response.code} - ${response.msg}`;
      console.group('Connection failed!')
      console.error(`corpus_analysis_init: ${errorText}`);
      client.notifyView('connecting-failed', { msg: errorText });
      console.groupEnd();
    }
  });
}

/**
 * Recieves meta data from the server via socket.io.
 */
function recieveMetaData(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('Client recieving meta data')
      console.info('corpus_analysis_meta_data: Client recieving meta data',
      'via socket.on');
      console.info(`corpus_analysis_meta_data: ${response.code} - ${response.msg}`);
      console.info(response);
      // executing the registered callbacks
      client.socketEventListeners[type].executeCallbacks(response.payload);
      console.groupEnd();
    } else {
      console.group('Failed to recieve meta data.');
      console.error('corpus_analysis_meta_data: Client failed to recieve',
                    'meta data via socket.on');
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.error(`corpus_analysis_meta_data: ${errorText}`);
      console.groupEnd();
    }
  });
}

/**
 * Recieves the query process status before any actual results are being
 * transmitted. So it recieves error codes if a query failed or
 * was invalid etc.
 * Also prepares the result.jsList for the incoming data.
 */
function recieveQueryStatus(type, client) {
  client.socket.on(type, (response) => {
    /**
     * Check if request for session was OK.
     * If OK execute registered callbacks and notify View.
     */
    if (response.code === 200) {
      console.group('corpus_analysis_query: Client recieving query process',
      'status via socket.on');
      console.info(`corpus_analysis_query: ${response.code} - ${response.msg}`);
      console.info(response);
      // executing the registered callbacks
      client.socketEventListeners[type].executeCallbacks(response.payload);
      console.groupEnd();
    } else {
      console.group('corpus_analysis_query: Client failed recieving',
                    'query process status via socket.on');
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      if (response.payload.code == 1281) {
        errorText += ' - Invalid Query';
      }
      console.error(`corpus_analysis_query: ${errorText}`);
      console.groupEnd();
    }
  });
}

/**
 * Recieves the query data from the request and handles it.
 */
function recieveQueryData(type, client) {
  /**
   * Check if request for session was OK.
   * If OK execute registered callbacks and notify View.
   */
  if (client.dynamicMode) {
    client.socket.on(type, (response) => {
    if (response.code === 200) {
      console.group('corpus_analysis_query_results: Recieveing query data')
      console.info('Client recieving query data via socket.on');
      console.info('Recieved chunk', response);
      /**
       * Execute registered callbacks and notify View.
       */
      client.socketEventListeners[type].executeCallbacks(response.payload);
      console.info('Added chunk data to results.data.');
      console.groupEnd();
    } else {
      console.group('corpus_analysis_query_results: Client failed recieving',
                    'the results via socket.on');
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      console.error(`corpus_analysis_query: ${errorText}`);
      console.groupEnd();
    }
    });
  } else {
    console.group('corpus_analysis_query_results: Loading query data.')
    console.info('Client loading imported query data from database.');
    // executing the registered callbacks
    client.socketEventListeners[type].executeCallbacks();
    console.groupEnd();
  }
}

// export listeners from this module
export {
  recieveConnected,
  recieveMetaData,
  recieveQueryStatus,
  recieveQueryData
};